// The PhotoshopActions plug-in
// Copyright (C) 2006-2007 Maurits Rijk
//
// ImageSizeEvent.cs
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

using System;
using System.Collections;

namespace Gimp.PhotoshopActions
{
  public class ImageSizeEvent : ActionEvent
  {
    [Parameter("Hght")]
    double _height;
    [Parameter("Wdth")]
    double _width;
    [Parameter("scaleStyles")]
    bool _scaleStyles;
    [Parameter("CnsP")]
    bool _constrainProportions;
    [Parameter("Intr")]
    EnumParameter _interpolation;

    protected override IEnumerable ListParameters()
    {
      DoubleParameter width = Parameters["Wdth"] as DoubleParameter;
      if (width != null)
	yield return width.Format();
      
      DoubleParameter height = Parameters["Hght"] as DoubleParameter;
      if (height != null)
	yield return height.Format();
      
      if (_interpolation != null)
	{
	  yield return Format(_interpolation, "Intr");
	}
    }
    
    override public bool Execute()
    {      
      DoubleParameter width = Parameters["Wdth"] as DoubleParameter; 
      if (width != null)
	_width = width.GetPixels(ActiveImage.Width);

      DoubleParameter height = Parameters["Hght"] as DoubleParameter; 
      if (height != null)
	_height = height.GetPixels(ActiveImage.Height);
      
      if (_constrainProportions)
	{
	  if (_width != 0 && _height == 0)
	    _height = ActiveImage.Height * _width / ActiveImage.Width;
	  if (_width == 0 && _height != 0)
	    _width = ActiveImage.Width * _height / ActiveImage.Height;
	}
      
      ActiveImage.Scale((int) _width, (int) _height);
      return true;
    }
  }
}
