// The PicturePackage plug-in
// Copyright (C) 2004-2007 Maurits Rijk
//
// DirImageProviderFactory.cs
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

using System;
using System.IO;

namespace Gimp.PicturePackage
{
  public class DirImageProviderFactory : ProviderFactory
  {
    readonly string[] _files;
    int _index = 0;

    public DirImageProviderFactory(string parent, bool recursive)
    {
      _files = Directory.GetFiles(parent);
    }

    public override void Reset()
    {
      _index = 0;
    }

    public override ImageProvider Provide()
    {
      while (_index < _files.Length)
	{
	  string file = _files[_index++];
	  
	  FileImageProvider provider = new FileImageProvider(file, file);
	  if (provider.GetImage() != null)
	    {
	      return provider;
	    }
	}
      return null;
    }

    public override void Cleanup(ImageProvider provider)
    {
      provider.Release();
    }
  }
}
